import * as React from 'react';
import {observer} from "mobx-react-lite";
import {
    ButtonField,
    FormProps,
    makeColumn,
    ModelDateField,
    ModelForm,
    ModelInput,
    ModelTable,
    ModelTableButton, PaddedPaper
} from "@rx-suite/ui";
import {ShippingMethodSelect} from "../orders/fields/ShippingMethodSelect";
import {CustomerOrderSelect} from "../orders/fields/CustomerOrderSelect";
import {StockOrderItemStore} from "../../stores/StockOrderItemStore";
import {t, Trans} from "@lingui/macro";
import {Container, Grid, Typography} from "@material-ui/core";
import {customerSelectionStore} from "../../hooks/useCustomerSelectionStore";
import {useStockOrderStore} from "../../hooks/useStockOrderStore";
import {StockOrderFrameDialog} from "./StockOrderFrameDialog";
import {StockOrderModel} from "../../stores/StockOrderStore";
import {StockOrderLensDialog} from "./StockOrderLensDialog";
import {Edit} from "@material-ui/icons";
import {StockItemEditDialog} from "./StockItemEditDialog";
import {ReleaseNumberInputField} from "../orders/fields/ReleaseNumberInputField";
import {useSnackbar} from "notistack";

export const StockOrderForm: React.FunctionComponent<FormProps> = observer(({model, readOnly}) => {
    const store = useStockOrderStore();
    const disableFields = !model.isNew
    const { enqueueSnackbar } = useSnackbar()
    const columns = React.useMemo(() => [
        makeColumn('item_identifier', t`Identifier`, 'string'),
        makeColumn('description', t`Decriptions`, 'string'),
        makeColumn('item_type', t`Item Type`, 'string'),
        makeColumn('quantity', t`Quantity`, 'string'),
    ], [model])

    React.useEffect(() => {
        if(model.isNew) {
            model?.nestedAssignment("customer_id", customerSelectionStore.currentCustomerId)
        } else {
            store.orderModel = model as StockOrderModel
        }
        store.submitErrors = "";

        return () => {
            store.hideFrameDialog();
            store.hideLensDialog();
        }
    }, [model, customerSelectionStore])

    const handleSubmit = React.useCallback(async () => {
        await store.submitOrder()

        if (store.submitErrors) {
            enqueueSnackbar(store.submitErrors, {variant: "error"})
        } else {
            enqueueSnackbar("Successfully created stock order", { variant: "success" })
        }
    }, [])

    return (
        <Container maxWidth={"xl"}>
            <PaddedPaper>
                <ModelForm model={model} readOnly={readOnly} suppressImportExport suppressMakeCopy maxWidth={"xl"} suppressWrapper>
                    <Grid container spacing={4}>
                        <Grid item xs={12}>
                            <Typography variant={"h4"}><Trans>Order Information</Trans></Typography>
                        </Grid>
                            {
                                model.viewModel.order_number &&
                                <Grid item xs={12}>
                                    <ModelInput name={'order_number'} disabled/>
                                </Grid>
                            }
                        <Grid item xs={12} lg={4}>
                            <ModelInput name={'reference_number'} disabled={disableFields}/>
                        </Grid>
                        <Grid item xs={12} lg={4}>
                            <ModelInput name={'po_number'} disabled={disableFields}/>
                        </Grid>
                        <Grid item xs={12} lg={4}>
                            <CustomerOrderSelect disabled={disableFields}/>
                        </Grid>
                        <Grid item xs={12} lg={4}>
                            <ModelDateField name={'date_ordered'} disabled timeFormat={false}/>
                        </Grid>
                        <Grid item xs={12} lg={4}>
                            <ModelDateField name={'date_completed'} disabled timeFormat={false}/>
                        </Grid>
                        <Grid item xs={12} lg={4}>
                            <ModelInput name={'status'} disabled />
                        </Grid>
                        <Grid item xs={12} lg={4}>
                            <ReleaseNumberInputField />
                        </Grid>
                        <Grid item xs={12}>
                            <Grid item xs={4}>
                                <ShippingMethodSelect />
                            </Grid>
                        </Grid>

                        <Grid item xs={12}>
                            <ButtonField text={t`Submit Order`} color={'secondary'} hidden={model.isNew || !model.isOpen} onClick={handleSubmit} loading={store.submittingOrder}/>
                        </Grid>
                        { store.submitErrors &&
                            <Grid item xs={12}>
                                <Typography variant={'body1'} color={'error'}><Trans>Error: {store.submitErrors}</Trans></Typography>
                            </Grid>
                        }
                    </Grid>
                </ModelForm>
                {
                    !model.isNew &&
                    <>
                        <Typography variant={'h4'}><Trans>Order Lines</Trans></Typography>
                        <ModelTable
                            store={StockOrderItemStore}
                            columns={columns}
                            showCreateButton={false}
                            suppressActionColumn={!model.isOpen}
                            suppressWrapper
                            defaultFiltered={[
                                {id: 'stock_order_id', value: model.getId()}
                            ]}
                            preTableActions={
                                <>
                                    <ButtonField text={'Add Frame'} onClick={store.showFrameDialog} hidden={!model.isOpen}/>
                                    <ButtonField text={`Add Lens`} onClick={store.showLensDialog} hidden={!model.isOpen}/>
                                </>
                            }
                            replacementActions={{
                                Edit: ({model: modelIn, store: storeIn}) => {
                                    return (
                                        <ModelTableButton
                                            store={storeIn}
                                            onClick={() => {
                                                store.applyStockItem(modelIn);
                                                store.showStockItemEditDialog();
                                            }}
                                            verb={"update"}
                                            disabled={false}
                                        >
                                            <Edit/>
                                        </ModelTableButton>
                                    )
                                },
                            }}
                        />
                    </>

                }
            </PaddedPaper>
            <StockOrderFrameDialog />
            <StockOrderLensDialog />
            <StockItemEditDialog />
        </Container>
    );
})