import {attribute, DataStore, FetchError, Model, model, schemas, Transport, Types} from '@rx-suite/data';
import {t} from "@lingui/macro";
import {string, array, object} from 'yup';
import {action, computed, observable, override, reaction, runInAction} from "mobx";
import {FrameTrace, JobTypeCode, PostalRegex} from "./_FieldConstants";
import * as moment from "moment";
import {RxUniverseStore} from "./RxUniverseStore";
import {PoValidator} from "../lib/PoValidator";
import {LabMaintenanceStore} from "./LabMaintenanceStore";

export type FrameStatusCode = "S" | "F" | "L" | "U" | "D" | "E" | "N";
export type RawData = { [key: string]: string };

@model({
    transportLayer: "/api/orders",
    permissionTarget: "Order",
    routingInfo: {
        list: '/orders',
        create: '/orders/create',
        read: '/orders/:id/read'
    }
})
export class OrderModel extends Model {
    static FRAME_KEYS = ["HBOX", "VBOX", "FED", "CIRC", "DBL"];

    static types: Types = {
        raw_data: "json",
        date_ordered: "date",
    }

    constructor() {
        super();
        super.setup(); // This registers the observable attributes and viewModel

        reaction(() => [this.raw_data, this.viewModel.raw_data, this.readOnly], () => {
            if(this.readOnly) {
                this.fixVCAValues();
            }
        })
    }

    @observable readOnly = true;

    @attribute({label: t`Order Number`, description: t`The unique identifier for the order. This number is automatically generated`})
    order_number = "";

    @attribute({label: t`Job Type`, description: t`What this job is producing`})
    job_type: JobTypeCode = "P";

    @attribute({label: t`Customer Num`, description: t`The account this order is being made under`})
    customer_num = "";

    @attribute({label: t`Patient Name`, description: t`The name of the patient this order is for`, validation: string().trim().required(t`Patient name is a required field`)})
    customer_name = "";

    @attribute({label: t`Customer City`, description: t`The shipping city for this patient`})
    customer_city = "";

    @attribute({label: t`Cust Job Id`})
    cust_job_id = "";

    @attribute({label: t`Date Completed`, description: t`The date this order was completed`})
    date_completed = "";

    @attribute({label: t`Date Received`, description: t`The date this order was received`})
    date_received = "";

    @attribute({label: t`Date Ordered`, description: t`The date this order was made`})
    date_ordered = moment();

    @attribute({label: t`Tray`, description: t`The tray number this order is associated with`})
    tray = "";

    @attribute({
        validation: string().trim().when("isVA", {
            is: true,
            then: schema => schema.label(t`Consult Number`).when('readOnly', {
                is: true,
                then: schema => schema.optional(),
                otherwise: schema => schema.required(t`Consult Number is a required field`)
            }),
            otherwise: schema => schema.optional().label(`PO Number`)
        }).test('already-exists', ({ label }) => t`${label} is already in use`, async (value, testContext) => {
            if(testContext.parent.readOnly) return true;

            return await PoValidator.instance.validate(value, testContext.parent.customer_id);

        })
    })
    po_number = "";

    @attribute({label: t`Ship To Num`})
    ship_to_num = "";

    @attribute({label: t`Ship To Name`, description: t`Name to print on the shipping label`})
    ship_to_name = "";

    @attribute({label: t`Ship To Address1`, description: t`Address to print on the shipping label. Cannot contain special characters except period(.) or dash(-)`})
    ship_to_address1 = "";

    @attribute({label: t`Ship To Address2`, description: t`Address to print on the shipping label. Cannot contain special characters except period(.) or dash(-)`})
    ship_to_address2 = "";

    @attribute({label: t`Ship To Address3`, description: t`Address to print on the shipping label. Cannot contain special characters except period(.) or dash(-)`})
    ship_to_address3 = "";

    @attribute({label: t`Ship To Address4`, description: t`Address to print on the shipping label. Cannot contain special characters except period(.) or dash(-)`})
    ship_to_address4 = "";

    // @ts-ignore
    @attribute({
        label: t`Ship To Code`,
        description: t`Zip or postal code`,
        validation: string().when('readOnly', {
            is: true,
            then: schema => schema.optional(),
            otherwise: schema => schema.matches(PostalRegex,  { message: t`Zip Code/Postal Code must be of valid format e.g., 12345, A1A 1A1, B2B2B2`, excludeEmptyString: true })
        })})
    ship_to_code = "";

    @attribute({
        label: t`Shipping Method`,
        description: t`The method used for shipping the order`,
        validation: string().nullable().test('shipping-method-required', t`Shipping method is a required field`, (value, testContext) => {
            if(!LabMaintenanceStore.selectedModel?.display_shipping_info || testContext.parent.readOnly) {
                return true;
            }

            return !!value;
        }),
    })
    shipping_method_id : string | null = "";

    @attribute({ autoForm: false, autoTable: false })
    shipping_method_code = "";

    @attribute({
        validation: string().trim().when('readOnly',{
            is: true,
            then: schema => schema.optional(),
            otherwise: schema => schema.when(
                "isVA", {
                    is: true,
                    then: schema => schema.length(4, t`Last 4 must be exactly 4 characters`),
                    otherwise: schema => schema.optional()
                }
            )
        })
    })
    release_number = "";

    @attribute()
    raw_data: RawData = {};

    @attribute({label: t`Type`, description: t`The type of the order`, validation: string().when('job_type', {
        is: (jobType : string) => ["F", "A"].includes(jobType),
        then: schema => schema.nullable().optional(),
        otherwise: schema => schema.required(t`Type is a required field`)
    })})
    style_type_id: string | null = "";

    @attribute({label: t`Style`, description: t`The style of the order - dependent on the order type`, validation: string().when('job_type', {
        is: (jobType : string) => ["F", "A"].includes(jobType),
        then: schema => schema.nullable().optional(),
        otherwise: schema => schema.required(t`Style is a required field`)
    })})
    style_id: string | null = "";

    @attribute({label: t`Material`, description: t`The material used for the order - depending on the style`, validation: string().when('job_type', {
        is: (jobType : string) => ["F", "A"].includes(jobType),
        then: schema => schema.nullable().optional(),
        otherwise: schema => schema.required(t`Material is a required field`)
    })})
    style_material_id: string | null = "";

    @attribute({label: t`Frame Status`, description: t`What kind of frame are we sending with this order, if any`})
    _fstatus: FrameStatusCode | null = "U";

    @attribute({label: t`Addons`, description: t`What addons should be applies to this order`,
        validation: array()
            .ensure().max(15, t`Must have less than or equal to 15 addons`)
            .when('job_type', {
                is: (jobType : string) => jobType === "A" ,
                then: schema => schema.ensure().min(1, t`Must add at least one addon`),
                otherwise: schema => schema.ensure().min(0)
    })})
    addon_ids: string[] = [];

    @attribute({ label: t`Hold At Lab?`, description: t`Whether to hold the order at the lab or not` })
    hold_at_lab = false;

    @attribute()
    quick_code_id: string | null = "";

    @attribute({label: t`Customer`, validation: string().required()})
    customer_id = "";

    @attribute()
    trace_id: string | null = "";

    @attribute()
    style_type_name = "";

    @attribute()
    style_name = "";

    @attribute()
    style_material_name = "";

    @attribute()
    fupc: string | null = "";

    @attribute()
    fram: string | null = "";

    @attribute()
    _fmupc: string | null = "";

    @attribute({label: t`Submitted By`})
    submitted_by: string | null = "";

    @attribute({label: t`Frame SKU`, validation: string().nullable().when('_fstatus', {
            is: (frameStatus : string) => frameStatus === "S",
            then: schema => schema.required(t`Must select a frame from Rx-Inventory`),
            otherwise: schema => schema.optional()
        })})
    frame_sku: string | null = "";

    @attribute()
    unvalid_message: string | null = "";

    @attribute({validation: string().max(385).optional().nullable()})
    comments: string | null = "";

    @attribute()
    is_valid = true;

    @attribute()
    job_track_station = "";

    @attribute()
    job_track_description = "";

    @attribute()
    is_shipped = false;

    @attribute()
    lens_color = "";

    @attribute()
    last_sync_time = "";

    @attribute()
    last_sync_message = "";

    @attribute()
    origin = "";

    @attribute()
    fcol = "";

    @attribute()
    lens_treat = "";

    @attribute()
    trace_file: any | null = null;

    @observable
    frameTrace: FrameTrace | null = null;

    // This runs when the Copy button is pressed
    @override
    fromExport(json: OrderModel) {
        this.resetView();
        this._new = true;
        this._clone = true;

        this.viewModel.updateFromJson(json, true);

        if (!this.isVA) {
            this.nestedAssignment("release_number", "");
        } //If Lab is VA keep Last 4 when copying

        if(this.traceID) {
            if(this.traceShouldBeCloned){
                this.getTrace();
            } else {
                this.nestedAssignment("raw_data.traceID", undefined);
            }
        }

        this.nestedAssignment("order_number", "");
        this.nestedAssignment("po_number", "");
        this.nestedAssignment("date_ordered", moment());
        this.nestedAssignment("raw_data.date_wanted", undefined);
        this.nestedAssignment("unvalid_message", "");
        this.nestedAssignment("is_valid", true);
    }

    @computed
    get traceID(): string | null | undefined  {
        return this.viewModel.dig("raw_data.traceID");
    }

    @computed
    get traceShouldBeCloned() {
        if(this.traceID) {
            const id = this.traceID;
            return !["INV", "FTD"].some(w => id.includes(w))
        } else {
            return false;
        }
    }

    @action
    async getTrace() {
        if(!this.traceID) return;

        const response = await Transport.fetch(`/api/rx_inventory_adapter/frame_traces/${encodeURIComponent(this.traceID.replace(/\./g, '&#46'))}`);
        const json = await response.json() as FrameTrace;

        if(json === null) {
            throw new FetchError(response, t`Could not find trace ${this.traceID} in RxUniverse`)
        }

        runInAction(() => {
            this.viewModel.frameTrace = json;
            this.nestedAssignment('trace_id', json.name);
            this.setFrameData({
                HBOX: json.a.toString(),
                VBOX: json.b.toString(),
                FED: json.ed.toString(),
                CIRC: json.circ.toString(),
                DBL: json.dbl.toString()
            });
        });
    }

    @computed
    get isVA() {
        return RxUniverseStore.selectedModel.isVA;
    }

    @computed
    get releaseNumRequired(){
        return RxUniverseStore.selectedModel.releaseNumRequired && !this.readOnly;
    }

    @computed get isAddonJob() {
        return this.viewModel.job_type === "A";
    }

    @computed get isFrameOnlyJob() {
        return this.viewModel.job_type === "F";
    }

    @computed get requiresEyeFields() {
        return !(this.isFrameOnlyJob || this.isAddonJob);
    }

    @computed get requiresJobFields() {
        return !this.isAddonJob;
    }

    @computed get isEdgeOnlyJob() {
        return this.viewModel.job_type === "E";
    }

    @computed get isInvalidJob() {
        return !this.is_valid;
    }

    @computed get hasComment() {
        if (this.comments) {
            return this.comments.trim().length > 0;
        }
        return false;
    }


      /* Since the form includes files, we have to provide a custom commit */
    @override
    async commit(): Promise<void> {
        this._state = 'updating';

        this.viewModel.submit();

        const formData = new FormData();
        formData.append('order', JSON.stringify(this.asJson));
        if (this.trace_file) {
            formData.append('trace_file', this.trace_file as Blob);
        }

        try {
            const response = await OrderStore.transportLayer.fetch(`/`, {
                method: 'POST',
                body: formData
            })
            return OrderStore.transportLayer.parseJson(response);
        } finally {
            runInAction(() => {
                this._state = 'done';
            });
        }
    }

    @action.bound
    clearPersonalizationFields() {
        const personalizationFields = ["ltPRVM1","ltPRVM2", "rtPRVM1","rtPRVM2", "ltMINCTR", "rtMINCTR", "ltMINEDG", "rtMINEDG"]

        personalizationFields.forEach( f =>{
            if(this.viewModel.dig(`raw_data.${f}`) == 0){
                this.nestedAssignment(`raw_data.${f}`, undefined)
            }
        })
    }

    @action.bound
    setReadOnly(readOnly: boolean) {
        this.readOnly = readOnly;
    }

    @action
    setFrameData(data: any) {
        this.viewModel.raw_data = {
            ...this.viewModel.raw_data,
            ...data
        }
    }

    @action
    fixVCAValues() {
        const data = this.viewModel.raw_data || {};
        const validKeys = OrderModel.FRAME_KEYS;

        Object.entries(data).forEach(([key, value]) => {
            if(validKeys.includes(key) || key.startsWith("lt") || key.startsWith("rt")) {
                const newVal = parseFloat(value);
                if(!isNaN(newVal)) {
                    data[key] =  newVal.toFixed(2);
                }
            }
        });

        this.viewModel.raw_data = data;
    }

    @computed
    get poNumberLabel() {
        return this.isVA ? t`Consult Number` : t`PO Number`
    }

    @computed
    get poNumberHint() {
        return {
            title: this.poNumberLabel,
            text: t`${this.isVA ? "Number used for external tracking" : "The Purchase Order number associated to this order. A number will be generated if omitted"}`
        }

    }
}

export const OrderStore = OrderModel.store as unknown as DataStore<OrderModel>;

const tmpModel = OrderStore.createModel();
const tmpSchema = tmpModel.validationSchema

export const createOrderValidationSchema = () => {
    return object().shape({}).concat(tmpSchema);
}

// @ts-ignore
schemas["OrderModel"] = createOrderValidationSchema();